@@ -8,7 +8,7 @@ use async_std::{
     io::{self, IoSlice, IoSliceMut, Read, Seek, SeekFrom, Write},
     task::{Context, Poll},
 };
-use cap_primitives::fs::is_file_read_write;
+use cap_primitives::{ambient_authority, fs::is_file_read_write, AmbientAuthority};
 use std::{fmt, pin::Pin};
 use unsafe_io::{AsUnsafeFile, OwnsRaw};
 #[cfg(windows)]
@@ -35,12 +35,12 @@ pub struct File {
 impl File {
     /// Constructs a new instance of `Self` from the given `async_std::fs::File`.
     ///
-    /// # Safety
+    /// # Ambient Authority
     ///
     /// `async_std::fs::File` is not sandboxed and may access any path that the host
     /// process has access to.
     #[inline]
-    pub unsafe fn from_std(std: fs::File) -> Self {
+    pub fn from_std(std: fs::File, _: AmbientAuthority) -> Self {
         Self { std }
     }
 
@@ -135,7 +135,7 @@ fn permissions_into_std(
 impl FromRawFd for File {
     #[inline]
     unsafe fn from_raw_fd(fd: RawFd) -> Self {
-        Self::from_std(fs::File::from_raw_fd(fd))
+        Self::from_std(fs::File::from_raw_fd(fd), ambient_authority())
     }
 }
 
@@ -143,7 +143,7 @@ impl FromRawFd for File {
 impl FromRawHandle for File {
     #[inline]
     unsafe fn from_raw_handle(handle: RawHandle) -> Self {
-        Self::from_std(fs::File::from_raw_handle(handle))
+        Self::from_std(fs::File::from_raw_handle(handle), ambient_authority())
     }
 }
 
