@@ -2,6 +2,8 @@ use crate::net::{Incoming, SocketAddr, TcpStream};
 #[cfg(unix)]
 use async_std::os::unix::io::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
 use async_std::{io, net};
+use cap_primitives::{ambient_authority, AmbientAuthority};
+use std::fmt;
 use unsafe_io::OwnsRaw;
 #[cfg(windows)]
 use {
@@ -14,11 +16,11 @@ use {
 /// This corresponds to [`async_std::net::TcpListener`].
 ///
 /// Note that this `TcpListener` has no `bind` method. To bind it to a socket
-/// address, you must first obtain a [`Catalog`] permitting the address, and
-/// then call [`Catalog::bind_tcp_listener`].
+/// address, you must first obtain a [`Pool`] permitting the address, and
+/// then call [`Pool::bind_tcp_listener`].
 ///
-/// [`Catalog`]: struct.Catalog.html
-/// [`Catalog::bind_tcp_listener`]: struct.Catalog.html#method.bind_tcp_listener
+/// [`Pool`]: struct.Pool.html
+/// [`Pool::bind_tcp_listener`]: struct.Pool.html#method.bind_tcp_listener
 pub struct TcpListener {
     std: net::TcpListener,
 }
@@ -26,12 +28,12 @@ pub struct TcpListener {
 impl TcpListener {
     /// Constructs a new instance of `Self` from the given `async_std::net::TcpListener`.
     ///
-    /// # Safety
+    /// # Ambient Authority
     ///
     /// `async_std::net::TcpListener` is not sandboxed and may access any address that the host
     /// process has access to.
     #[inline]
-    pub unsafe fn from_std(std: net::TcpListener) -> Self {
+    pub fn from_std(std: net::TcpListener, _: AmbientAuthority) -> Self {
         Self { std }
     }
 
@@ -53,7 +55,7 @@ impl TcpListener {
         self.std
             .accept()
             .await
-            .map(|(tcp_stream, addr)| (unsafe { TcpStream::from_std(tcp_stream) }, addr))
+            .map(|(tcp_stream, addr)| (TcpStream::from_std(tcp_stream, ambient_authority()), addr))
     }
 
     /// Returns an iterator over the connections being received on this listener.
@@ -62,7 +64,7 @@ impl TcpListener {
     #[inline]
     pub fn incoming(&self) -> Incoming {
         let incoming = self.std.incoming();
-        unsafe { Incoming::from_std(incoming) }
+        Incoming::from_std(incoming, ambient_authority())
     }
 
     // async_std doesn't have `TcpListener::set_ttl`.
@@ -78,7 +80,7 @@ impl TcpListener {
 impl FromRawFd for TcpListener {
     #[inline]
     unsafe fn from_raw_fd(fd: RawFd) -> Self {
-        Self::from_std(net::TcpListener::from_raw_fd(fd))
+        Self::from_std(net::TcpListener::from_raw_fd(fd), ambient_authority())
     }
 }
 
@@ -86,7 +88,10 @@ impl FromRawFd for TcpListener {
 impl FromRawSocket for TcpListener {
     #[inline]
     unsafe fn from_raw_socket(socket: RawSocket) -> Self {
-        Self::from_std(net::TcpListener::from_raw_socket(socket))
+        Self::from_std(
+            net::TcpListener::from_raw_socket(socket),
+            ambient_authority(),
+        )
     }
 }
 
@@ -141,4 +146,8 @@ impl IntoRawHandleOrSocket for TcpListener {
 /// Safety: `TcpListener` wraps a `net::TcpListener` which owns its handle.
 unsafe impl OwnsRaw for TcpListener {}
 
-// TODO: impl Debug for TcpListener
+impl fmt::Debug for TcpListener {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.std.fmt(f)
+    }
+}
