@@ -2,6 +2,8 @@ use crate::net::{Ipv4Addr, Ipv6Addr, SocketAddr};
 #[cfg(unix)]
 use async_std::os::unix::io::{AsRawFd, FromRawFd, IntoRawFd, RawFd};
 use async_std::{io, net};
+use cap_primitives::{ambient_authority, AmbientAuthority};
+use std::fmt;
 use unsafe_io::OwnsRaw;
 #[cfg(windows)]
 use {
@@ -15,14 +17,14 @@ use {
 ///
 /// Note that this `UdpSocket` has no `bind`, `connect`, or `send_to` methods. To
 /// create a `UdpSocket` bound to an address or to send a message to an address,
-/// you must first obtain a [`Catalog`] permitting the address, and then call
-/// [`Catalog::bind_udp_socket`], or [`Catalog::connect_udp_socket`], or
-/// [`Catalog::send_to_udp_socket_addr`].
+/// you must first obtain a [`Pool`] permitting the address, and then call
+/// [`Pool::bind_udp_socket`], or [`Pool::connect_udp_socket`], or
+/// [`Pool::send_to_udp_socket_addr`].
 ///
-/// [`Catalog`]: struct.Catalog.html
-/// [`Catalog::bind_udp_socket`]: struct.Catalog.html#method.bind_udp_socket
-/// [`Catalog::connect_udp_socket`]: struct.Catalog.html#method.connect_udp_socket
-/// [`Catalog::send_to_udp_socket_addr`]: struct.Catalog.html#method.send_to_udp_socket_addr
+/// [`Pool`]: struct.Pool.html
+/// [`Pool::bind_udp_socket`]: struct.Pool.html#method.bind_udp_socket
+/// [`Pool::connect_udp_socket`]: struct.Pool.html#method.connect_udp_socket
+/// [`Pool::send_to_udp_socket_addr`]: struct.Pool.html#method.send_to_udp_socket_addr
 pub struct UdpSocket {
     pub(crate) std: net::UdpSocket,
 }
@@ -30,12 +32,12 @@ pub struct UdpSocket {
 impl UdpSocket {
     /// Constructs a new instance of `Self` from the given `async_std::net::UdpSocket`.
     ///
-    /// # Safety
+    /// # Ambient Authority
     ///
     /// `async_std::net::UdpSocket` is not sandboxed and may access any address that the host
     /// process has access to.
     #[inline]
-    pub unsafe fn from_std(std: net::UdpSocket) -> Self {
+    pub fn from_std(std: net::UdpSocket, _: AmbientAuthority) -> Self {
         Self { std }
     }
 
@@ -219,7 +221,7 @@ impl UdpSocket {
 impl FromRawFd for UdpSocket {
     #[inline]
     unsafe fn from_raw_fd(fd: RawFd) -> Self {
-        Self::from_std(net::UdpSocket::from_raw_fd(fd))
+        Self::from_std(net::UdpSocket::from_raw_fd(fd), ambient_authority())
     }
 }
 
@@ -227,7 +229,7 @@ impl FromRawFd for UdpSocket {
 impl FromRawSocket for UdpSocket {
     #[inline]
     unsafe fn from_raw_socket(socket: RawSocket) -> Self {
-        Self::from_std(net::UdpSocket::from_raw_socket(socket))
+        Self::from_std(net::UdpSocket::from_raw_socket(socket), ambient_authority())
     }
 }
 
@@ -282,4 +284,8 @@ impl IntoRawHandleOrSocket for UdpSocket {
 /// Safety: `UdpSocket` wraps a `net::UdpSocket` which owns its handle.
 unsafe impl OwnsRaw for UdpSocket {}
 
-// TODO: impl Debug for UdpSocket
+impl fmt::Debug for UdpSocket {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        self.std.fmt(f)
+    }
+}
